﻿using System;
using System.Windows.Forms;
using FileArrangement.App.Properties;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.App
{
	public partial class ConditionEditForm : Form
	{
		public ConditionEditForm()
		{
			InitializeComponent();

			WindowState = FormWindowState.Maximized;
		}

		public HostHelper HostHelper { get; set; }
		public FormHelper FormHelper { get; set; }
		private Controller ConditionController { get; set; }

		public ICondition Result { get; private set; }

		private void ChangeCondition()
		{
			ICondition condition = FormHelper.ShowSelectConditionForm();

			if ( condition != null )
			{
				SetCondition( condition );
			}
		}

		public void SetCondition( ICondition condition )
		{
			if ( ConditionController != null )
			{
				ConditionPanel.Controls.Clear();
				ConditionController.Control.Dispose();
			}

			ConditionController = FormHelper.Factory.Create( condition );

			Control control = ConditionController.Control;
			control.Dock = DockStyle.Fill;

			ConditionPanel.Controls.Add( ConditionController.Control );
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			if ( ConditionController != null )
			{
				if ( !ConditionController.IsValidControlValues() )
				{
					MessageBox.Show( "無効な値が含まれています。" );
					return;
				}

				ConditionController.UpdateToTarget();

				Result = (ICondition) ConditionController.Target;
			}

			DialogResult = DialogResult.OK;
		}

		private void okCancelPanel1_CancelButton_Click( object sender, EventArgs e )
		{
			DialogResult = DialogResult.Cancel;
		}

		private void ConditionEditForm_Load( object sender, EventArgs e )
		{
			RootTableLayoutPanel.Dock = DockStyle.Fill;
			ConditionPanel.Dock = DockStyle.Fill;
			okCancelPanel1.Dock = DockStyle.Fill;

			ChangeConditionToolStripButton.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText;
			CopyToolStripButton.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText;
			PasteChangeToolStripButton.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText;
			SetUpIcons();
		}

		private void SetUpIcons()
		{
			ChangeConditionToolStripButton.Image = Resources.note_l_b.ToBitmap();
			CopyToolStripButton.Image = Resources.copy_b.ToBitmap();
			PasteChangeToolStripButton.Image = Resources.paste_bl_1_b.ToBitmap();
		}

		private void ChangeConditionToolStripButton_Click( object sender, EventArgs e )
		{
			ChangeCondition();
		}

		private void CopyToolStripButton_Click( object sender, EventArgs e )
		{
			if ( ConditionController != null )
			{
				if ( !ConditionController.IsValidControlValues() )
				{
					MessageBox.Show( "無効な値が含まれています。" );
					return;
				}

				ConditionController.UpdateToTarget();

				FormHelper.ClipBoard.Value = ConditionController.Target;
			}
		}

		private void PasteChangeToolStripButton_Click( object sender, EventArgs e )
		{
			if ( FormHelper.ClipBoard.Value is ICondition )
			{
				ICondition condition = (ICondition) FormHelper.ClipBoard.Value;

				SetCondition( condition.Clone() );
			}
		}
	}
}