﻿using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.App
{
	public partial class EditForm : Form
	{
		private readonly ControllerCollection Controllers = new ControllerCollection();

		private bool Initialized;

		public EditForm()
		{
			InitializeComponent();
		}

		public FormHelper FormHelper { get; set; }

		public void SetTarget( IEnumerable value )
		{
			if ( Initialized )
			{
				throw new InvalidOperationException( "SetTargetは2度呼び出さないで下さい" );
			}

			foreach ( object target in value )
			{
				Control content = CreateContent( target );

				ContentsFlowLayoutPanel.Controls.Add( content );
			}

			Initialized = true;
		}

		private void UpdateToTarget()
		{
			Controllers.UpdateToTargetAll();
		}

		private Control CreateContent( object target )
		{
			Panel panel = new Panel { AutoSize = true, BorderStyle = BorderStyle.FixedSingle };

			Controller controller = FormHelper.Factory.Create( target );

			panel.Controls.Add( controller.Control );

			Controllers.Add( controller );

			return panel;
		}

		private void EditForm_Load( object sender, EventArgs e )
		{
			RootTableLayoutPanel.Dock = DockStyle.Fill;
			ContentsFlowLayoutPanel.Dock = DockStyle.Fill;
			okCancelPanel1.Dock = DockStyle.Fill;

			ContentsFlowLayoutPanel.FlowDirection = FlowDirection.TopDown;
			ContentsFlowLayoutPanel.WrapContents = false;
			ContentsFlowLayoutPanel.AutoScroll = true;
			ContentsFlowLayoutPanel.MinimumSize = new Size( 500, 400 );
		}

		private void okCancelPanel1_OKButton_Click( object sender, EventArgs e )
		{
			if ( !Controllers.IsValidControllerValuesAll() )
			{
				MessageBox.Show( "無効な値が含まれています。" );
				return;
			}

			UpdateToTarget();

			DialogResult = DialogResult.OK;
		}

		private void okCancelPanel1_CancelButton_Click( object sender, EventArgs e )
		{
			DialogResult = DialogResult.Cancel;
		}
	}
}