﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using FileArrangement.Plugins;
using Microsoft.VisualBasic;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.App
{
	public class FormHelper
	{
		private readonly HostHelper HostHelper;
		private MyClipBoard ClipBoardStore = new MyClipBoard();

		public FormHelper( HostHelper helper )
		{
			HostHelper = helper;

			SetUp();
		}

		public MyClipBoard ClipBoard
		{
			get { return ClipBoardStore; }
			set { ClipBoardStore = value; }
		}

		public ControllerFactory Factory { get; private set; }

		public string ShowNameInputBox( string prompt )
		{
			string input = Interaction.InputBox(
				prompt,
				"名前を入力して下さい",
				string.Empty,
				-1,
				-1
				);

			return input.Trim();
		}

		public bool ShowConfirmBox( IWin32Window owner, string message )
		{
			DialogResult dialogResult = MessageBox.Show(
				owner,
				message,
				"確認",
				MessageBoxButtons.OKCancel,
				MessageBoxIcon.Question,
				MessageBoxDefaultButton.Button1
				);

			return dialogResult == DialogResult.OK;
		}

		private void SetUp()
		{
			SetUpFactory();
		}

		private void SetUpFactory()
		{
			Factory = new TypeRegisterControllerFactory(
				typeof ( NormalTextCondition ), typeof ( NormalTextConditionController ) );

			ControllerFactory current = Factory;

			current = SetUpTypeRegisterFactories( HostHelper.ConditionTypeInformations, current );
			current = SetUpTypeRegisterFactories( HostHelper.EffectTypeInformations, current );
			current = SetUpTypeRegisterFactories( HostHelper.GeneralTypeInformations, current );

			current = current.SetNext( new TypeRegisterControllerFactory(
			                           	typeof ( RegExCondition ), typeof ( RegExConditionController ) ) );
			current = current.SetNext( new TypeRegisterControllerFactory(
			                           	typeof ( TrueCondition ), typeof ( TrueConditionController ) ) );
			current = current.SetNext( new TypeRegisterControllerFactory(
			                           	typeof ( FalseCondition ), typeof ( FalseConditionController ) ) );

			current = current.SetNext( new AndConditionControllerFactory( this ) );
			current = current.SetNext( new OrConditionControllerFactory( this ) );
			current = current.SetNext( new NotConditionControllerFactory( this ) );

			current.SetNext( new DefaultControllerFactory() );
		}

		private ControllerFactory SetUpTypeRegisterFactories( List<TypeInformation> informations, ControllerFactory current )
		{
			foreach ( TypeInformation information in informations )
			{
				Type controllerType = GetControllerTypeWithAttribute( information.Type );

				if ( controllerType != null )
				{
					TypeRegisterControllerFactory factory = new TypeRegisterControllerFactory(
						information.Type, controllerType );

					current = current.SetNext( factory );
				}
			}
			return current;
		}

		private Type GetControllerTypeWithAttribute( Type target )
		{
			object[] attributes = target.GetCustomAttributes(
				typeof ( ControllerSpecificationAttribute ),
				false );

			if ( attributes.Length == 0 )
			{
				return null;
			}

			ControllerSpecificationAttribute result =
				(ControllerSpecificationAttribute) attributes[0];

			return result.Controller;
		}

		public ICondition ShowSelectConditionForm()
		{
			ICondition result = null;

			using ( TypeInformationSelectForm selectForm = new TypeInformationSelectForm() )
			{
				selectForm.SetTypeInformations( HostHelper.ConditionTypeInformations );
				selectForm.Text = "条件選択";

				if ( selectForm.ShowDialog() ==
				     DialogResult.OK )
				{
					if ( selectForm.Result.Count == 1 )
					{
						result = (ICondition) selectForm.Result[0].Create();
					}
				}
			}

			return result;
		}

		public IEffect[] ShowSelectEffectForm()
		{
			using ( TypeInformationSelectForm selectForm = new TypeInformationSelectForm() )
			{
				selectForm.SetTypeInformations( HostHelper.EffectTypeInformations );
				selectForm.Text = "効果選択";
				selectForm.SelectionMode = SelectionMode.MultiExtended;

				if ( selectForm.ShowDialog() ==
				     DialogResult.OK )
				{
					List<IEffect> list = new List<IEffect>();

					foreach ( TypeInformation information in selectForm.Result )
					{
						list.Add( (IEffect) information.Create() );
					}

					return list.ToArray();
				}
			}

			return new IEffect[] { };
		}

		public void ShowTypeInformations( List<TypeInformation> list, string caption )
		{
			using ( TypeInformationsListUpForm form = new TypeInformationsListUpForm() )
			{
				form.Initialize( list );
				form.Text = caption;

				form.ShowDialog();
			}
		}
	}
}