﻿using System;
using System.Drawing;
using System.Windows.Forms;
using FileArrangement.App.Properties;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.App
{
	public class NotConditionController : Controller
	{
		private Controller ConditionController;
		private Panel ConditionPanel;

		public NotConditionController( object target, FormHelper formHelper )
			: base( target )
		{
			FormHelper = formHelper;
		}

		private NotCondition CastedTarget
		{
			get { return (NotCondition) Target; }
		}

		public FormHelper FormHelper { get; private set; }

		private FlowLayoutPanel Root { get; set; }

		public override void UpdateToTarget()
		{
			if ( ConditionController != null )
			{
				ConditionController.UpdateToTarget();

				ICondition condition = (ICondition) ConditionController.Target;

				CastedTarget.Condition = condition.Clone();
			}
		}

		public override void UpdateToControl()
		{
			if ( CastedTarget.Condition != null )
			{
				SetCondition( CastedTarget.Condition.Clone() );
			}
		}

		public override bool IsValidControlValues()
		{
			if ( ConditionController != null )
			{
				return ConditionController.IsValidControlValues();
			}

			return true;
		}

		protected override Control CreateControl()
		{
			Root = new FlowLayoutPanel
			       {
			       	FlowDirection = FlowDirection.TopDown,
			       	WrapContents = false,
			       	AutoSize = true,
			       	AutoScroll = true,
			       	MinimumSize = new Size( 500, 200 )
			       };

			Label titleLabel = new Label
			                   {
			                   	Text = "Not",
			                   	BackColor = Color.White,
			                   	AutoSize = true
			                   };

			ToolStrip manipulationToolStrip = CreateManipulationToolStrip();

			ConditionPanel = new Panel { AutoSize = true };

			Root.Controls.Add( titleLabel );
			Root.Controls.Add( manipulationToolStrip );
			Root.Controls.Add( ConditionPanel );

			return Root;
		}

		private ToolStrip CreateManipulationToolStrip()
		{
			ToolStrip result = new ToolStrip();

			ToolStripButton changeToolStripButton = new ToolStripButton
			                                        {
			                                        	Text = "変更",
			                                        	Image = Resources.note_l_b.ToBitmap()
			                                        };
			changeToolStripButton.Click += changeToolStripButton_Click;

			ToolStripButton copyToolStripButton = new ToolStripButton
			                                      {
			                                      	Text = "コピー",
			                                      	Image = Resources.copy_b.ToBitmap()
			                                      };
			copyToolStripButton.Click += copyToolStripButton_Click;

			ToolStripButton pasteChangeToolStripButton = new ToolStripButton
			                                             {
			                                             	Text = "貼り付け変更",
			                                             	Image = Resources.paste_bl_1_b.ToBitmap()
			                                             };
			pasteChangeToolStripButton.Click += pasteButton_Click;

			result.Items.Add( changeToolStripButton );
			result.Items.Add( copyToolStripButton );
			result.Items.Add( pasteChangeToolStripButton );
			return result;
		}

		private void pasteButton_Click( object sender, EventArgs e )
		{
			if ( FormHelper.ClipBoard.Value is ICondition )
			{
				ICondition condition = (ICondition) FormHelper.ClipBoard.Value;

				SetCondition( condition.Clone() );
			}
		}

		private void changeToolStripButton_Click( object sender, EventArgs e )
		{
			ICondition condition = FormHelper.ShowSelectConditionForm();

			if ( condition != null )
			{
				SetCondition( condition );
			}
		}

		private void copyToolStripButton_Click( object sender, EventArgs e )
		{
			if ( ConditionController != null )
			{
				ConditionController.UpdateToTarget();
				ICondition condition = (ICondition) ConditionController.Target;
				FormHelper.ClipBoard.Value = condition.Clone();
			}
		}

		private void SetCondition( ICondition condition )
		{
			if ( ConditionController != null )
			{
				ConditionPanel.Controls.Clear();
				ConditionController.Control.Dispose();
			}

			ConditionController = FormHelper.Factory.Create( condition );
			ConditionPanel.Controls.Add( ConditionController.Control );
		}
	}
}