﻿using System.Windows.Forms;
using FileArrangement.Plugins;
using Tomham.IO.FileManipulation;
using Tomham.Windows.Forms;

namespace FileArrangement.App
{
	public abstract class TextConditionController : AutoAdjustedController<TextCondition>
	{
		private string TitleTextStore = string.Empty;

		protected TextConditionController( object target ) : base( target )
		{
		}

		protected string TitleText
		{
			get { return TitleTextStore; }
			set { TitleTextStore = value; }
		}

		protected LabeledComboBox PatternLabeledComboBox { get; set; }
		private CheckBox IgnoreCaseCheckBox { get; set; }
		private CheckBox OnlyNameMatchingCheckBox { get; set; }

		public override void UpdateToControl()
		{
			PatternLabeledComboBox.ComboBox.Text = CastedTarget.Pattern;
			IgnoreCaseCheckBox.Checked = CastedTarget.IgnoreCase;
			OnlyNameMatchingCheckBox.Checked = CastedTarget.OnlyNameMatching;

			LoadPatternHistory();
		}

		public override void UpdateToTarget()
		{
			CastedTarget.Pattern = PatternLabeledComboBox.ComboBox.Text;
			CastedTarget.IgnoreCase = IgnoreCaseCheckBox.Checked;
			CastedTarget.OnlyNameMatching = OnlyNameMatchingCheckBox.Checked;

			SavePatternHistory();
		}

		protected abstract void SavePatternHistory();

		protected abstract void LoadPatternHistory();

		protected override void SetUpContentControls()
		{
			Label titleLabel = new Label
			                   {
			                   	Text = TitleText,
			                   	AutoSize = true
			                   };

			PatternLabeledComboBox = new LabeledComboBox { TitleText = "パターン" };

			PatternLabeledComboBox.ComboBox.Text = CastedTarget.Pattern;
			PatternLabeledComboBox.Width = 400;
			PatternLabeledComboBox.ComboBox.AutoCompleteMode = AutoCompleteMode.None;
			PatternLabeledComboBox.ComboBox.AutoCompleteSource = AutoCompleteSource.ListItems;

			IgnoreCaseCheckBox = new CheckBox { Text = "大文字と小文字の違いを無視する", AutoSize = true, Checked = CastedTarget.IgnoreCase };

			OnlyNameMatchingCheckBox = new CheckBox
			                           {
			                           	Text = "名前のみを対象とする",
			                           	AutoSize = true,
			                           	Checked = CastedTarget.OnlyNameMatching
			                           };

			AddContent( titleLabel );

			AddContent( PatternLabeledComboBox );
			AddContent( IgnoreCaseCheckBox );
			AddContent( OnlyNameMatchingCheckBox );
		}
	}
}