﻿using NUnit.Framework;
using Tomham;

namespace FileArrangement.Plugins.Common.Test
{
	[TestFixture]
	public class RegExMacroEffectTest : AssertionHelper
	{
		private RegExMacroEffect CreateIgnoreCaseTestEffect()
		{
			RegExMacroEffect effect = new RegExMacroEffect();
			effect.Pattern = @"(file\d)";
			return effect;
		}

		private RunningWithFileArguments CreateArguments()
		{
			RunningWithFileArguments arguments = new RunningWithFileArguments(
				new ConsoleMessageOutput(),
				true,
				@"U:\Programming\Projects\FileArrangement\FileArrangement.Plugins.Common.Test\bin\Debug\Dir\File1.txt",
				@"C:Work\<1>" );

			return arguments;
		}

		private RegExMacroEffect CreateOnlyNameMatchingTestEffect()
		{
			RegExMacroEffect effect = new RegExMacroEffect();
			effect.Pattern = @"\\(dir)\\";
			return effect;
		}

		[Test]
		public void Create()
		{
			RegExMacroEffect effect = new RegExMacroEffect();

			Expect( effect.IgnoreCase, Is.True );
			Expect( effect.OnlyNameMatching, Is.True );
		}

		[Test]
		public void IgnoreCaseIsFalse()
		{
			RunningWithFileArguments arguments = CreateArguments();

			RegExMacroEffect effect = CreateIgnoreCaseTestEffect();
			effect.IgnoreCase = false;

			effect.RunningWithFile( arguments );

			Expect( arguments.DestinationDirectory, Is.EqualTo( @"C:Work\<1>" ) );
		}

		[Test]
		public void IgnoreCaseIsTrue()
		{
			RunningWithFileArguments arguments = CreateArguments();

			RegExMacroEffect effect = CreateIgnoreCaseTestEffect();
			effect.IgnoreCase = true;

			effect.RunningWithFile( arguments );

			Expect( arguments.DestinationDirectory, Is.EqualTo( @"C:Work\File1" ) );
		}

		[Test]
		public void OnlyNameMatchingIsFalse()
		{
			RunningWithFileArguments arguments = CreateArguments();

			RegExMacroEffect effect = CreateOnlyNameMatchingTestEffect();
			effect.OnlyNameMatching = false;

			effect.RunningWithFile( arguments );

			Expect( arguments.DestinationDirectory, Is.EqualTo( @"C:Work\Dir" ) );
		}

		[Test]
		public void OnlyNameMatchingIsTrue()
		{
			RunningWithFileArguments arguments = CreateArguments();

			RegExMacroEffect effect = CreateOnlyNameMatchingTestEffect();
			effect.OnlyNameMatching = true;

			effect.RunningWithFile( arguments );

			Expect( arguments.DestinationDirectory, Is.EqualTo( @"C:Work\<1>" ) );
		}
	}
}