﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using Tomham.IO;

namespace FileArrangement.Plugins.Common
{
	[ControllerSpecification( typeof ( RegExMacroEffectController ) )]
	[Information(
		"正規表現マクロ",
		"前方参照を利用するマクロです。例: <1>, <2>, ...<n>",
		"tom_ham",
		"1.00" )]
	[Serializable]
	public class RegExMacroEffect : MacroEffect
	{
		private readonly Dictionary<string, string> MacroValuePairs = new Dictionary<string, string>();
		private bool IgnoreCaseStore = true;
		private bool OnlyNameMatchingStore = true;

		public bool IgnoreCase
		{
			get { return IgnoreCaseStore; }
			set { IgnoreCaseStore = value; }
		}

		public string Pattern { get; set; }

		public bool OnlyNameMatching
		{
			get { return OnlyNameMatchingStore; }
			set { OnlyNameMatchingStore = value; }
		}

		protected override void Prepare( RunningWithFileArguments arguments )
		{
			MacroValuePairs.Clear();

			string input;

			if ( OnlyNameMatching )
			{
				input = PathUtils.GetName( arguments.Source );
			}
			else
			{
				input = Path.GetFullPath( arguments.Source );
			}

			RegexOptions options = RegexOptions.None;

			if ( IgnoreCase )
			{
				options = RegexOptions.IgnoreCase;
			}

			Match match = Regex.Match( input, Pattern, options );

			if ( 2 <= match.Groups.Count )
			{
				for ( int i = 1; i < match.Groups.Count; i++ )
				{
					MacroValuePairs.Add( i.ToString(), match.Groups[i].Value );
				}
			}
			else
			{
				arguments.Cancel = true;
			}
		}

		public override IEffect Clone()
		{
			return (IEffect) MemberwiseClone();
		}

		protected override string Calculate( string macro, string sourcePath )
		{
			string value;

			if ( MacroValuePairs.TryGetValue( macro, out value ) )
			{
				return value;
			}

			throw new ApplicationException();
		}

		protected override IEnumerable<string> GetMacros()
		{
			return MacroValuePairs.Keys;
		}
	}
}