﻿using System.Windows.Forms;
using Tomham.Windows.Forms;

namespace FileArrangement.Plugins.Common
{
	public class RegExMacroEffectController : AutoAdjustedController<RegExMacroEffect>
	{
		public RegExMacroEffectController( object target ) : base( target )
		{
		}

		private LabeledComboBox PatternLabeledComboBox { get; set; }
		private CheckBox IgnoreCaseCheckBox { get; set; }
		private CheckBox OnlyNameMatchingCheckBox { get; set; }

		public override void UpdateToControl()
		{
			PatternLabeledComboBox.ComboBox.Text = CastedTarget.Pattern;
			IgnoreCaseCheckBox.Checked = CastedTarget.IgnoreCase;
			OnlyNameMatchingCheckBox.Checked = CastedTarget.OnlyNameMatching;
		}

		public override void UpdateToTarget()
		{
			CastedTarget.Pattern = PatternLabeledComboBox.ComboBox.Text;
			CastedTarget.IgnoreCase = IgnoreCaseCheckBox.Checked;
			CastedTarget.OnlyNameMatching = OnlyNameMatchingCheckBox.Checked;

			if ( CastedTarget.Pattern != string.Empty &&
			     !RegExMacroEffectControllerHelper.History.Contains( CastedTarget.Pattern ) )
			{
				RegExMacroEffectControllerHelper.History.Insert( 0, CastedTarget.Pattern );
			}
		}

		protected override void SetUpContentControls()
		{
			Label titleLabel = new Label
			                   {
			                   	Text = "正規表現マクロ",
			                   	AutoSize = true
			                   };

			PatternLabeledComboBox = new LabeledComboBox { TitleText = "パターン" };

			PatternLabeledComboBox.ComboBox.Text = CastedTarget.Pattern;
			PatternLabeledComboBox.Width = 400;
			PatternLabeledComboBox.ComboBox.AutoCompleteMode = AutoCompleteMode.None;
			PatternLabeledComboBox.ComboBox.AutoCompleteSource = AutoCompleteSource.ListItems;
			PatternLabeledComboBox.ComboBox.Items.AddRange( RegExMacroEffectControllerHelper.History.ToArray() );

			IgnoreCaseCheckBox = new CheckBox { Text = "大文字と小文字の違いを無視する", AutoSize = true, Checked = CastedTarget.IgnoreCase };

			OnlyNameMatchingCheckBox = new CheckBox
			                           {
			                           	Text = "名前のみを対象とする",
			                           	AutoSize = true,
			                           	Checked = CastedTarget.OnlyNameMatching
			                           };

			AddContent( titleLabel );

			AddContent( PatternLabeledComboBox );
			AddContent( IgnoreCaseCheckBox );
			AddContent( OnlyNameMatchingCheckBox );
		}
	}
}