﻿using System.Drawing;
using System.Windows.Forms;

namespace FileArrangement.Plugins.Common
{
	public class SameFileNameWorkEffectController : AutoAdjustedController<SameFileNameWorkEffect>
	{
		public SameFileNameWorkEffectController( object target ) : base( target )
		{
		}

		private GroupBox UnWantedFileManipulationGroupBox { get; set; }
		private GroupBox WantedFileManipulationGroupBox { get; set; }
		private CheckBox HasCheckedLastWriteTimeCheckBox { get; set; }
		private RadioButton CancelRadioButton { get; set; }
		private RadioButton DeleteSourceRadioButton { get; set; }
		private RadioButton NoneRadioButton { get; set; }
		private RadioButton ReNameRadioButton { get; set; }

		public override void UpdateToTarget()
		{
			if ( NoneRadioButton.Checked )
			{
				CastedTarget.WantedFileManipulation = WantedFileManipulation.None;
			}
			else
			{
				CastedTarget.WantedFileManipulation = WantedFileManipulation.ReName;
			}

			if ( CancelRadioButton.Checked )
			{
				CastedTarget.UnWantedFileManipulation = UnWantedFileManipulation.Cancel;
			}
			else
			{
				CastedTarget.UnWantedFileManipulation = UnWantedFileManipulation.DeleteSource;
			}

			CastedTarget.HasCheckedLastWriteTime = HasCheckedLastWriteTimeCheckBox.Checked;
		}

		public override void UpdateToControl()
		{
			switch ( CastedTarget.WantedFileManipulation )
			{
				case WantedFileManipulation.None:
					NoneRadioButton.Checked = true;
					break;

				case WantedFileManipulation.ReName:
					ReNameRadioButton.Checked = true;
					break;
			}

			switch ( CastedTarget.UnWantedFileManipulation )
			{
				case UnWantedFileManipulation.DeleteSource:
					DeleteSourceRadioButton.Checked = true;
					break;

				case UnWantedFileManipulation.Cancel:
					CancelRadioButton.Checked = true;
					break;
			}

			HasCheckedLastWriteTimeCheckBox.Checked = CastedTarget.HasCheckedLastWriteTime;
		}

		protected override void SetUpContentControls()
		{
			Label titleLabel = new Label
			                   {
			                   	Text = "同名ファイルの処理",
			                   	AutoSize = true
			                   };

			WantedFileManipulationGroupBox = new GroupBox { Text = "必要と判断したファイルの処理", AutoSize = true };
			UnWantedFileManipulationGroupBox = new GroupBox { Text = "不要と判断したファイルの処理", AutoSize = true };

			NoneRadioButton = new RadioButton { Text = "上書き", Location = new Point( 20, 20 ) };
			ReNameRadioButton = new RadioButton { Text = "リネーム", Location = new Point( 150, 20 ) };

			CancelRadioButton = new RadioButton { Text = "キャンセル", Location = new Point( 20, 20 ) };
			DeleteSourceRadioButton = new RadioButton
			                          {
			                          	Text = "削除",
			                          	AutoSize = true,
			                          	Location = new Point( 150, 20 )
			                          };

			HasCheckedLastWriteTimeCheckBox = new CheckBox { Text = "バイナリが違い、かつ最終更新日時が新しいファイルのみ必要と判断する", AutoSize = true };

			UnWantedFileManipulationGroupBox.Controls.Add( CancelRadioButton );
			UnWantedFileManipulationGroupBox.Controls.Add( DeleteSourceRadioButton );

			WantedFileManipulationGroupBox.Controls.Add( NoneRadioButton );
			WantedFileManipulationGroupBox.Controls.Add( ReNameRadioButton );

			AddContent( titleLabel );
			AddContent( WantedFileManipulationGroupBox );
			AddContent( UnWantedFileManipulationGroupBox );
			AddContent( HasCheckedLastWriteTimeCheckBox );
		}
	}
}