﻿using System.Windows.Forms;
using Tomham.Windows.Forms.AutoGeneration;

namespace FileArrangement.Plugins
{
	public abstract class AutoAdjustedController<T> : Controller
	{
		private const int Margin = 3;
		private Panel Root;

		protected AutoAdjustedController( object target ) : base( target )
		{
		}

		protected T CastedTarget
		{
			get { return (T) Target; }
		}

		protected Panel ContentsPanel { get; private set; }

		public override bool IsValidControlValues()
		{
			return true;
		}

		protected override Control CreateControl()
		{
			Root = new Panel();

			ContentsPanel = new Panel();

			Root.Controls.Add( ContentsPanel );

			SetUpContentControls();

			return Root;
		}

		protected abstract void SetUpContentControls();

		protected void AddContent( Control content )
		{
			ContentsPanel.Controls.Add( content );
			AdjustControls();
		}

		protected void RemoveContent( Control content )
		{
			ContentsPanel.Controls.Remove( content );
			content.Dispose();
			AdjustControls();
		}

		protected void AdjustControls()
		{
			AdjustControl( ContentsPanel, Margin );
			AdjustControl( Root, Margin );
		}

		private void AdjustControl( Control control, int margin )
		{
			int top = margin;

			foreach ( Control child in control.Controls )
			{
				child.Left = margin;
				child.Top = top;
				top += child.Height + margin;
			}

			control.Height = top;

			int width = 50;

			foreach ( Control child in control.Controls )
			{
				if ( width < child.Width )
				{
					width = child.Width;
				}
			}

			control.Width = width + margin;
		}
	}
}