﻿using System.Collections.Generic;

namespace FileArrangement.Plugins
{
	public abstract class MacroEffect : IEffect
	{
		#region IEffect Members

		public virtual void RunningWithRule( RunningWithRuleArguments arguments )
		{
		}

		public virtual void RanWithRule( RanWithRuleArguments arguments )
		{
		}

		public void RunningWithFile( RunningWithFileArguments arguments )
		{
			Prepare( arguments );

			arguments.DestinationDirectory =
				Replace( arguments.DestinationDirectory, arguments.Source );
		}

		public void ConditionMatching( ConditionMatchingArguments arguments )
		{
			arguments.NewName = Replace( arguments.NewName, arguments.Source );
		}

		public virtual void RanWithFile( RanWithFileArguments arguments )
		{
		}

		public abstract IEffect Clone();

		#endregion

		protected virtual void Prepare( RunningWithFileArguments arguments )
		{
		}

		private string Replace( string target, string source )
		{
			foreach ( string macro in GetMacros() )
			{
				string oldValue = string.Format( "<{0}>", macro );

				if ( target.Contains( oldValue ) )
				{
					string newValue = Calculate( macro, source );

					target = target.Replace( oldValue, newValue );
				}
			}

			return target;
		}

		protected abstract string Calculate( string macro, string sourcePath );
		protected abstract IEnumerable<string> GetMacros();
	}
}