﻿using NUnit.Extensions.Converter;
using NUnit.Framework;
using Tomham;
using Tomham.IO.FileManipulation;

namespace FileArrangement.Test
{
	[TestFixture]
	public class RuleSetRunnerTest : AssertionHelper
	{
		#region Setup/Teardown

		[SetUp]
		public void SetUp()
		{
		}

		#endregion

		private readonly RuleSetRunner Runner = CreateRunner();

		private static RuleSetRunner CreateRunner()
		{
			IMessageOutput output = new ConsoleMessageOutput();

			IRuleSetRunnerMode mode = new PreviewMode( output );

			return new RuleSetRunner( mode );
		}

		private string source = @"U:\Work\FileArrangement.TestsTemp";
		private string destination = @"U:\Work\UnExistsDirectory";

		private readonly Converter MyConverter =
			new BeanValueConverter(
				new DefaultConverter(),
				typeof ( ProcessResult ),
				"Source",
				"Destination" );

		private RuleSet GetRuleSet()
		{
			Rule rule = new Rule();

			rule.Destination = destination;
			rule.Name = "Test";

			RegExCondition condition = new RegExCondition();

			condition.Pattern = @".*";

			rule.Condition = condition;

			RuleSet ruleSet = new RuleSet();
			ruleSet.SourceDirectories.Add( source );
			ruleSet.Rules.Add( rule );
			ruleSet.Name = "TestRuleSet";

			return ruleSet;
		}

		[Test]
		public void TargetEntryIsFilesRecursive()
		{
			RuleSet ruleSet = GetRuleSet();

			ruleSet.TargetEntry = TargetEntry.FilesRecursive;

			RunResult result = Runner.Run( ruleSet );

			object expected = E.a(
				E.a( @"U:\Work\FileArrangement.TestsTemp\File1.txt",
				     @"U:\Work\UnExistsDirectory\File1.txt" ),
				E.a( @"U:\Work\FileArrangement.TestsTemp\File2.txt",
				     @"U:\Work\UnExistsDirectory\File2.txt" ),
				E.a( @"U:\Work\FileArrangement.TestsTemp\SubDir\cat.txt",
				     @"U:\Work\UnExistsDirectory\cat.txt" ),
				E.a( @"U:\Work\FileArrangement.TestsTemp\SubDir\dog.txt",
				     @"U:\Work\UnExistsDirectory\dog.txt" ) );

			MyConverter.AssertEquals( expected, result.ProcessResults );
		}

		[Test]
		public void TargetEntryIsFiles()
		{
			RuleSet ruleSet = GetRuleSet();

			ruleSet.TargetEntry = TargetEntry.Files;

			RunResult result = Runner.Run( ruleSet );

			object expected = E.a(
				E.a( @"U:\Work\FileArrangement.TestsTemp\File1.txt",
				     @"U:\Work\UnExistsDirectory\File1.txt" ),
				E.a( @"U:\Work\FileArrangement.TestsTemp\File2.txt",
				     @"U:\Work\UnExistsDirectory\File2.txt" ) );

			MyConverter.AssertEquals( expected, result.ProcessResults );
		}

		[Test]
		public void CreatedDirectories()
		{
			RuleSet ruleSet = GetRuleSet();
			ruleSet.Rules.Clear();

			Rule rule = new Rule { Condition = new TrueCondition(), Destination = @"U:\Work\UnExistsDirectory\A" };
			ruleSet.Rules.Add( rule  );

			RunResult result = Runner.Run( ruleSet );

			object expected = E.a( @"U:\Work\UnExistsDirectory", @"U:\Work\UnExistsDirectory\A" );

			MyConverter.AssertEquals( expected, result.CreatedDirectories );

			rule.Destination = @"U:\Work";
			result = Runner.Run( ruleSet );
			MyConverter.AssertEquals( 0, result.CreatedDirectories.Count );
		}
	}
}