﻿using System.IO;
using System.Reflection;
using log4net;
using Tomham;
using Tomham.IO;

namespace FileArrangement
{
	public class MoveMode : IRuleSetRunnerMode
	{
		private readonly ILog LogStore;

		public MoveMode( IMessageOutput output )
		{
			LogStore = LogManager.GetLogger( MethodBase.GetCurrentMethod().DeclaringType );

			Output = output;
		}

		#region IRuleSetRunnerMode Members

		public IMessageOutput Output { get; set; }

		public ILog Log
		{
			get { return LogStore; }
		}

		public void CreateDirectory( string path )
		{
			Directory.CreateDirectory( path );

			string message = string.Format( "ディレクトリの作成: {0}", PathUtils.DoubleQuotString( path ) );

			Output.WriteLine( message );
			Log.Info( message );
		}

		public void Process( string source, string destination )
		{
			FileUtils.ForceMoveFile( source, destination );

			string message = string.Format(
				"{0} -> {1}",
				PathUtils.DoubleQuotString( source ),
				PathUtils.DoubleQuotString( destination ) );

			Output.WriteLine( message );
			Log.Info( message );
		}

		public void Running( RuleSet ruleSet )
		{
			Output.WriteLine( string.Format( "実行開始: {0}", ruleSet.Name ) );
		}

		public void Ran( RuleSet ruleSet, RunResult runResult )
		{
			Output.WriteLine( string.Format( "処理エントリ数: {0}", runResult.ProcessResults.Count ) );
			Output.WriteLine( string.Format( "実行終了: {0}", ruleSet.Name ) );
		}

		#endregion
	}
}