﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using FileArrangement.Plugins;

namespace FileArrangement
{
	public class PluginHelper
	{
		public static TypeInformation[] GetTypeInformations( string directoryPath, string interfaceName )
		{
			if ( !Directory.Exists( directoryPath ) )
			{
				throw new DirectoryNotFoundException( directoryPath );
			}

			List<TypeInformation> reslt = new List<TypeInformation>();

			string[] dlls =
				Directory.GetFiles( directoryPath, "*.dll" );

			foreach ( string dll in dlls )
			{
				try
				{
					Assembly assembly = Assembly.LoadFrom( dll );

					Type[] types = assembly.GetTypes();

					foreach ( Type t in types )
					{
						if ( IsTarget( t, interfaceName ) )
						{
							reslt.Add( CreateTypeInformation( t, dll ) );
						}
					}
				}
				catch ( Exception e )
				{
					Console.WriteLine( e.Message );
				}
			}

			return reslt.ToArray();
		}

		private static bool IsTarget( Type type, string expectedInterfaceName )
		{
			return type.IsClass &&
			       type.IsPublic &&
			       !type.IsAbstract &&
			       type.GetInterface( expectedInterfaceName ) != null &&
			       HasDefaultConstructor( type );
		}

		private static bool HasDefaultConstructor( Type type )
		{
			ConstructorInfo info = type.GetConstructor( Type.EmptyTypes );
			return info != null;
		}

		private static TypeInformation CreateTypeInformation( Type type, string location )
		{
			object[] attributes =
				type.GetCustomAttributes( typeof ( InformationAttribute ), false );

			if ( attributes.Length != 0 )
			{
				InformationAttribute information = (InformationAttribute) attributes[0];

				return new TypeInformation( type,
				                            information.Name,
				                            information.Description,
				                            information.Author,
				                            information.Version,
				                            location );
			}

			return new TypeInformation( type, type.FullName, string.Empty, string.Empty, string.Empty, location );
		}
	}
}