﻿using System;
using System.Collections.Generic;
using FileArrangement.Plugins;
using Tomham.IO.FileManipulation;

namespace FileArrangement
{
	[Serializable]
	public class Rule
	{
		private ICondition ConditionStore;
		private string DestinationStore;
		private List<IEffect> EffectsStore;
		private string NameStore;
		private string NewNamePatternStore;

		public string Name
		{
			get
			{
				if ( NameStore == null )
				{
					NameStore = string.Empty;
				}

				return NameStore;
			}
			set { NameStore = value; }
		}

		public string Destination
		{
			get
			{
				if ( DestinationStore == null )
				{
					DestinationStore = string.Empty;
				}

				return DestinationStore;
			}
			set { DestinationStore = value; }
		}

		public ICondition Condition
		{
			get
			{
				if ( ConditionStore == null )
				{
					ConditionStore = new AndCondition();
				}

				return ConditionStore;
			}
			set { ConditionStore = value; }
		}

		public List<IEffect> Effects
		{
			get
			{
				if ( EffectsStore == null )
				{
					EffectsStore = new List<IEffect>();
				}

				return EffectsStore;
			}

			set { EffectsStore = value; }
		}

		public string NewNamePattern
		{
			get
			{
				if ( NewNamePatternStore == null )
				{
					NewNamePatternStore = string.Empty;
				}

				return NewNamePatternStore;
			}
			set { NewNamePatternStore = value; }
		}

		public void Adjust()
		{
			Name = Name.Trim();
			Destination = Destination.Trim();
		}

		public bool IsValid()
		{
			if (
				Destination.Trim() ==
				string.Empty )

			{
				return false;
			}

			return true;
		}

		public Rule Clone()
		{
			Rule clone = new Rule
			             {
			             	Name = Name,
			             	Destination = Destination,
							NewNamePattern= NewNamePattern,
			             	Condition = Condition.Clone()
			             };

			foreach ( IEffect effect in Effects )
			{
				clone.Effects.Add( effect.Clone() );
			}

			return clone;
		}

		public override string ToString()
		{
			return Name;
		}
	}
}