﻿using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using FileArrangement.Plugins;
using Tomham.Collections.Specialized;
using Tomham.IO;

namespace FileArrangement
{
	public class TextConditionControllerHelper : IGeneral
	{
		private static List<string> NormalTextConditionPatternHistoryStore;
		private static List<string> RegExConditionPatternHistoryStore;
		private readonly string NormalTextConditionPatternHistoryFileName;
		private readonly string RegExConditionPatternHistoryFileName;

		public TextConditionControllerHelper()
		{
			NormalTextConditionPatternHistoryFileName = GetFileName( "NormalTextConditionPatternHistory.dat" );
			RegExConditionPatternHistoryFileName = GetFileName( "RegExConditionPatternHistory.dat" );
		}

		public static List<string> RegExConditionPatternHistory
		{
			get
			{
				if ( RegExConditionPatternHistoryStore == null )
				{
					RegExConditionPatternHistoryStore = new List<string>();
				}

				return RegExConditionPatternHistoryStore;
			}
			set { RegExConditionPatternHistoryStore = value; }
		}

		public static List<string> NormalTextConditionPatternHistory
		{
			get
			{
				if ( NormalTextConditionPatternHistoryStore == null )
				{
					NormalTextConditionPatternHistoryStore = new List<string>();
				}

				return NormalTextConditionPatternHistoryStore;
			}
			set { NormalTextConditionPatternHistoryStore = value; }
		}

		#region IGeneral Members

		public void Load()
		{
			LoadHistory( NormalTextConditionPatternHistoryFileName, NormalTextConditionPatternHistory );
			LoadHistory( RegExConditionPatternHistoryFileName, RegExConditionPatternHistory );
		}

		public void UnLoad()
		{
			SaveHistory( NormalTextConditionPatternHistoryFileName, NormalTextConditionPatternHistory );
			SaveHistory( RegExConditionPatternHistoryFileName, RegExConditionPatternHistory );
		}

		#endregion

		private void SaveHistory( string fileName, IEnumerable<string> history )
		{
			StringList list = new StringList();

			list.AddRange( history );

			list.SaveToFile( fileName, Encoding.UTF8 );
		}

		private void LoadHistory( string fileName, ICollection<string> history )
		{
			if ( File.Exists( fileName ) )
			{
				StringList list = new StringList( fileName, Encoding.UTF8 );

				for ( int i = 0; i < list.Count; i++ )
				{
					if ( 50 <= i )
					{
						break;
					}

					history.Add( list[i] );
				}
			}
		}

		private string GetFileName( string name )
		{
			Assembly assembly = Assembly.GetExecutingAssembly();

			DirectoryInfo info = Directory.GetParent( assembly.Location );

			return PathUtils.Combine( info.FullName, name );
		}
	}
}