﻿using NUnit.Framework;

namespace Tomham.IO.FileManipulation.Test
{
	[TestFixture]
	public class RegExConditionTest
	{
		#region Setup/Teardown

		[SetUp]
		public void SetUp()
		{
			TestHelper.SetUp();
		}

		#endregion

		[Test]
		public void IgnoreCaseIsFalse()
		{
			TextCondition condition = new RegExCondition
			                               {
			                               	Pattern = @"ple.*TXT",
			                               	IgnoreCase = false
			                               };

			Assert.IsFalse( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void IgnoreCaseIsTrue()
		{
			TextCondition condition = new RegExCondition
			                               {
			                               	Pattern = @"ple.*TXT",
			                               	IgnoreCase = true
			                               };

			Assert.IsTrue( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void Create()
		{
			TextCondition condition = new RegExCondition();

			Assert.IsTrue( condition.IgnoreCase );
			Assert.IsTrue( condition.OnlyNameMatching );
		}

		[Test]
		public void OnlyNameMattingIsFalse()
		{
			TextCondition condition = new RegExCondition
			{
				Pattern = "bin",
				OnlyNameMatching = false
			};

			Assert.IsTrue( condition.IsMatch( TestHelper.GetArguments() ) );
		}

		[Test]
		public void OnlyNameMattingIsTrue()
		{
			TextCondition condition = new RegExCondition
			{
				Pattern = "bin",
				OnlyNameMatching = true
			};

			Assert.IsFalse( condition.IsMatch( TestHelper.GetArguments() ) );
		}
	}
}