using System;
using System.IO;
using System.Text;
using Microsoft.Win32;

namespace Tomham.IO
{
	/// <summary>
	/// FileSystem⏕邽߂Static Method񋟂
	/// </summary>
	public static class FileUtils
	{
		public static bool FileOrDirectoryExists( string path )
		{
			return ( File.Exists( path ) || Directory.Exists( path ) );
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="path1"></param>
		/// <param name="path2"></param>
		/// <returns>0菬 path1LastWriteTime &lt; path2LastWriteTime</returns>
		public static int CompareLastWriteTime( string path1, string path2 )
		{
			if ( path1 == null )
			{
				throw new ArgumentNullException( "path1" );
			}

			if ( path2 == null )
			{
				throw new ArgumentNullException( "path2" );
			}

			if ( ! FileOrDirectoryExists( path1 ) )
			{
				throw new ArgumentException( "path1݂܂" );
			}

			if ( ! FileOrDirectoryExists( path2 ) )
			{
				throw new ArgumentException( "path2݂܂" );
			}

			return DateTime.Compare(
				File.GetLastWriteTime( path1 ),
				File.GetLastWriteTime( path2 )
				);
		}

		/// <summary>
		/// 悪ǂݎpłv
		/// </summary>
		/// <param name="sourcePath"></param>
		/// <param name="destPath"></param>
		public static void ForceMoveFile( string sourcePath, string destPath )
		{
			if ( File.Exists( destPath ) )
			{
				ForceCopyFile( sourcePath, destPath );
				ForceDeleteFile( sourcePath );
			}
			else
			{
				File.Move( sourcePath, destPath );
			}
		}

		/// <summary>
		/// 悪ǂݎpłv
		/// </summary>
		/// <param name="sourcePath"></param>
		/// <param name="destPath"></param>
		public static void ForceCopyFile( string sourcePath, string destPath )
		{
			if ( File.Exists( destPath ) )
			{
				if ( HasSystemAttribute( destPath ) ||
				     HasReadOnlyAttribute( destPath ) ||
				     HasHiddenAttribute( destPath )
					)
				{
					ChangeAttributesToWritable( destPath );
				}
			}

			File.Copy( sourcePath, destPath, true );
		}

		public static void CopyAttributes( string sourcePath, string destPath )
		{
			FileAttributes sourceAttributes = File.GetAttributes( sourcePath );
			File.SetAttributes( destPath, sourceAttributes );
		}

		public static bool HasReadOnlyAttribute( string path )
		{
			return ( File.GetAttributes( path ) & FileAttributes.ReadOnly ) ==
			       FileAttributes.ReadOnly;
		}

		public static bool HasHiddenAttribute( string path )
		{
			return ( File.GetAttributes( path ) & FileAttributes.Hidden ) ==
			       FileAttributes.Hidden;
		}

		public static bool HasSystemAttribute( string path )
		{
			return ( File.GetAttributes( path ) & FileAttributes.System ) ==
			       FileAttributes.System;
		}

		/// <summary>
		/// ύXď㏑\ɂ
		/// </summary>
		/// <param name="path"></param>
		/// <returns></returns>
		public static void ChangeAttributesToWritable( string path )
		{
			int tmp = Convert.ToInt32( File.GetAttributes( path ) );

			if ( HasReadOnlyAttribute( path ) )
			{
				tmp -= Convert.ToInt32( FileAttributes.ReadOnly );
			}

			if ( HasHiddenAttribute( path ) )
			{
				tmp -= Convert.ToInt32( FileAttributes.Hidden );
			}

			if ( HasSystemAttribute( path ) )
			{
				tmp -= Convert.ToInt32( FileAttributes.System );
			}

			FileAttributes newAttributes = (FileAttributes) ( tmp );

			File.SetAttributes( path, newAttributes );
		}

		/// <summary>
		/// ǂݎptĂt@Cł폜
		/// </summary>
		/// <param name="path"></param>
		public static void ForceDeleteFile( string path )
		{
			if ( path == null )
			{
				throw new ArgumentNullException( "path" );
			}

			if ( Directory.Exists( path ) )
			{
				throw new ArgumentException( "pathDirectorył" );
			}

			if ( !File.Exists( path ) )
			{
				return;
			}

			ChangeAttributesToWritable( path );
			File.Delete( path );
		}

		/// <summary>
		/// ǂݎpƂAg̓Directorył폜
		/// </summary>
		/// <param name="path"></param>
		public static void ForceDeleteDirectory( string path )
		{
			string[] fileList = Directory.GetFiles( path );

			foreach ( string file in fileList )
			{
				ForceDeleteFile( file );
			}

			string[] subDirList = Directory.GetDirectories( path );

			foreach ( string subDir in subDirList )
			{
				ForceDeleteDirectory( subDir );
			}

			ChangeAttributesToWritable( path );
			Directory.Delete( path, true );
		}

		public static void ForceDeleteFileOrDirectory( string path )
		{
			if ( Directory.Exists( path ) )
			{
				ForceDeleteDirectory( path );
			}
			else
			{
				ForceDeleteFile( path );
			}
		}

		public static void RegisterExtension( string extension,
		                                      string path,
		                                      string fileType,
		                                      string description )
		{
			StringBuilder commandline = new StringBuilder();

			commandline.Append( PathUtils.DoubleQuotString( path ) );
			commandline.Append( " " );
			commandline.Append( PathUtils.DoubleQuotString( "%1" ) );

			//t@C^Cvo^
			RegistryKey regkey =
				Registry.ClassesRoot.CreateSubKey(
					extension );
			regkey.SetValue( "", fileType );
			regkey.Close();

			//t@C^CvƂ̐o^
			RegistryKey shellkey =
				Registry.ClassesRoot.CreateSubKey(
					fileType );
			shellkey.SetValue( "", description );

			// c
			RegistryKey restKey =
				shellkey.CreateSubKey( "shell\\open\\command" );
			restKey.SetValue( "", commandline.ToString() );
			restKey.Close();
		}

		public static void UnRegisterExtension( string extension,
		                                        string fileType )
		{
			Registry.ClassesRoot.DeleteSubKeyTree( extension );
			Registry.ClassesRoot.DeleteSubKeyTree( fileType );
		}
	}
}