﻿using System.Windows.Forms;
using NUnit.Extensions.Converter;
using NUnit.Framework;

namespace Tomham.Windows.Forms.Tests
{
	[TestFixture]
	public class ListBoxUtilitiesTest : AssertionHelper
	{
		private readonly ListBoxConverter converter = new ListBoxConverter( new DefaultConverter() );

		private ListBox CreateListBox()
		{
			ListBox listBox = new ListBox();

			listBox.BeginUpdate();

			listBox.Items.AddRange(
				new object[] { "ant", "bird", "cat", "dog", "eagle", "fly" } );

			listBox.SetSelected( 1, true );
			listBox.SetSelected( 3, true );
			listBox.SetSelected( 4, true );

			listBox.EndUpdate();

			return listBox;
		}

		[Test]
		public void MoveDownSelectedItems()
		{
			ListBox actual = CreateListBox();

			object expected = E.a( "ant", "cat", "*bird", "fly", "*dog", "*eagle" );

			actual.MoveDownSelectedItems();

			converter.AssertEquals( expected, actual, 1 );

			expected = E.a( "ant", "cat", "fly", "*bird", "*dog", "*eagle" );

			actual.MoveDownSelectedItems();

			converter.AssertEquals( expected, actual, 1 );

			expected = E.a( "ant", "cat", "fly", "*bird", "*dog", "*eagle" );

			actual.MoveDownSelectedItems();

			converter.AssertEquals( expected, actual, 1 );
		}

		[Test]
		public void MoveUpSelectedItems()
		{
			ListBox actual = CreateListBox();

			object expected = E.a( "*bird", "ant", "*dog", "*eagle", "cat", "fly" );

			actual.MoveUpSelectedItems();

			converter.AssertEquals( expected, actual, 1 );

			expected = E.a( "*bird", "*dog", "*eagle", "ant", "cat", "fly" );

			actual.MoveUpSelectedItems();

			converter.AssertEquals( expected, actual, 1 );

			expected = E.a( "*bird", "*dog", "*eagle", "ant", "cat", "fly" );

			actual.MoveUpSelectedItems();

			converter.AssertEquals( expected, actual, 1 );
		}

		[Test]
		public void MoveUpSelectedItemsWithNoItem()
		{
			ListBox noItemListBox = new ListBox();

			noItemListBox.MoveUpSelectedItems();

			Assert.AreEqual( 0, noItemListBox.Items.Count );
		}

		[Test]
		public void SwapItems()
		{
			ListBox actual = CreateListBox();

			actual.SwapItems( 0, 4 );

			object expected = E.a( "*eagle", "*bird", "cat", "*dog", "ant", "fly" );

			converter.AssertEquals( expected, actual, 1 );

			actual.SwapItems( 0, 5 );

			expected = E.a( "fly", "*bird", "cat", "*dog", "ant", "*eagle" );

			converter.AssertEquals( expected, actual, 1 );
		}
	}
}