﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

namespace Tomham.Windows.Forms
{

	// Usage:
	// まずはSetSource
	// Sourceに変更があればUpdateMatchedItemsを呼んで表示の更新をする
	// フィルタ方法を変えたいならIsMatchedItemを設定
	// Itemのソート方法はSetMatchedItemsSorterで変更できる
	// Itemを得るにはGetMatchedItemsとかを使う

	public partial class FilterListBox : UserControl
	{
		public FilterListBox()
		{
			InitializeComponent();
		}

		#region Properties

		private Predicate<object> IsMatchedItemStore;
		private int MaxShowItemsCountStore;

		[Category( "Appearance" )]
		[Browsable( true )]
		public int MaxShowItemsCount
		{
			get { return MaxShowItemsCountStore; }
			set
			{
				MaxShowItemsCountStore = value;
				UpdateMatchedItems();
			}
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public string FilterText
		{
			get { return textBoxFilter.Text; }
			set { textBoxFilter.Text = value; }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public Color TextForeColor
		{
			get { return textBoxFilter.ForeColor; }
			set { SetTextForeColor( value ); }
		}

		[Category( "Appearance" )]
		[Browsable( true )]
		public Color TextBackColor
		{
			get { return textBoxFilter.BackColor; }
			set { SetTextBackColor( value ); }
		}

		[Category( "Behavior" )]
		[Browsable( true )]
		public bool HorizontalScrollbar
		{
			get { return listBoxMatchedItems.HorizontalScrollbar; }
			set { listBoxMatchedItems.HorizontalScrollbar = value; }
		}

		[Category( "Behavior" )]
		[Browsable( true )]
		public int HorizontalExtent
		{
			get { return listBoxMatchedItems.HorizontalExtent; }
			set { listBoxMatchedItems.HorizontalExtent = value; }
		}

		[Category( "Behavior" )]
		[Browsable( true )]
		public SelectionMode SelectionMode
		{
			get { return listBoxMatchedItems.SelectionMode; }
			set { listBoxMatchedItems.SelectionMode = value; }
		}

		[Browsable( false )]
		public bool TextBoxFocused
		{
			get { return textBoxFilter.Focused; }
		}

		[Browsable( false )]
		public bool ListBoxFoucused
		{
			get { return listBoxMatchedItems.Focused; }
		}

		[Category( "Behavior" )]
		[Browsable( true )]
		public ContextMenuStrip TextBoxContextMenuStrip
		{
			get { return textBoxFilter.ContextMenuStrip; }
			set { textBoxFilter.ContextMenuStrip = value; }
		}

		[Category( "Behavior" )]
		[Browsable( true )]
		public ContextMenuStrip ListBoxContextMenuStrip
		{
			get { return listBoxMatchedItems.ContextMenuStrip; }
			set { listBoxMatchedItems.ContextMenuStrip = value; }
		}

		[Browsable( false )]
		public Predicate<object> IsMatchedItem
		{
			get { return IsMatchedItemStore; }
			set
			{
				IsMatchedItemStore = value;

				UpdateMatchedItems();
			}
		}

		#region Source

		private IEnumerable SourceStore = new object[0];

		public IEnumerable GetSource()
		{
			return SourceStore;
		}

		public void SetSource( IEnumerable value )
		{
			if ( value == null )
			{
				SourceStore = new object[0];
			}
			else
			{
				SourceStore = value;
			}

			UpdateMatchedItems();
		}

		#endregion

		private void SetTextForeColor( Color value )
		{
			textBoxFilter.ForeColor = value;
			listBoxMatchedItems.ForeColor = value;
		}

		private void SetTextBackColor( Color value )
		{
			textBoxFilter.BackColor = value;
			listBoxMatchedItems.BackColor = value;
		}

		#endregion Properties

		#region MatchedItemsSorter

		private IComparer MatchedItemsSorterStore;

		public IComparer GetMatchedItemsSorter()
		{
			if ( MatchedItemsSorterStore == null )
			{
				return DefaultFilterListBoxItemComparer.Instance;
			}

			return MatchedItemsSorterStore;
		}

		public void SetMatchedItemsSorter( IComparer comparer )
		{
			MatchedItemsSorterStore = comparer;

			UpdateMatchedItems();
		}

		#endregion

		#region ListBox Click Events

		[Category( "Action" )]
		[Browsable( true )]
		public event EventHandler ListBoxClick;

		protected virtual void OnListBoxClick( EventArgs e )
		{
			if ( ListBoxClick != null )
			{
				ListBoxClick( this, e );
			}
		}

		[Category( "Action" )]
		[Browsable( true )]
		public event EventHandler ListBoxDoubleClick;

		protected virtual void OnListBoxDoubleClick( EventArgs e )
		{
			if ( ListBoxDoubleClick != null )
			{
				ListBoxDoubleClick( this, e );
			}
		}

		[Category( "Action" )]
		[Browsable( true )]
		public event MouseEventHandler ListBoxMouseClick;

		protected virtual void OnListBoxMouseClick( MouseEventArgs e )
		{
			if ( ListBoxMouseClick != null )
			{
				ListBoxMouseClick( this, e );
			}
		}

		[Category( "Action" )]
		[Browsable( true )]
		public event MouseEventHandler ListBoxMouseDoubleClick;

		protected virtual void OnListBoxMouseDoubleClick( MouseEventArgs e )
		{
			if ( ListBoxMouseDoubleClick != null )
			{
				ListBoxMouseDoubleClick( this, e );
			}
		}

		private void listBoxMatchedItems_Click( object sender, EventArgs e )
		{
			OnListBoxClick( e );
		}

		private void listBoxMatchedItems_DoubleClick( object sender, EventArgs e )
		{
			OnListBoxDoubleClick( e );
		}

		private void listBoxMatchedItems_MouseClick( object sender, MouseEventArgs e )
		{
			OnListBoxMouseClick( e );
		}

		private void listBoxMatchedItems_MouseDoubleClick( object sender, MouseEventArgs e )
		{
			OnListBoxMouseDoubleClick( e );
		}

		#endregion ListBox Click Events

		#region FilterTextChanged Event

		private void textBoxFilter_TextChanged( object sender, EventArgs e )
		{
			UpdateMatchedItems();

			OnFilterTextChanged( e );
		}

		[Category( "Default" )]
		[Browsable( true )]
		public event EventHandler FilterTextChanged;

		protected virtual void OnFilterTextChanged( EventArgs e )
		{
			if ( FilterTextChanged != null )
			{
				FilterTextChanged( this, e );
			}
		}

		#endregion

		#region Key Events

		#region  Key Events with TextBoxFilter

		private readonly Keys selectAllKey = Keys.Control | Keys.A;

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyPressEventHandler TextBoxKeyPress;

		protected virtual void OnTextBoxKeyPress( KeyPressEventArgs e )
		{
			if ( TextBoxKeyPress != null )
			{
				TextBoxKeyPress( this, e );
			}
		}

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyEventHandler TextBoxKeyDown;

		protected virtual void OnTextBoxKeyDown( KeyEventArgs e )
		{
			if ( TextBoxKeyDown != null )
			{
				TextBoxKeyDown( this, e );
			}
		}

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyEventHandler TextBoxKeyUp;

		protected virtual void OnTextBoxKeyUp( KeyEventArgs e )
		{
			if ( TextBoxKeyUp != null )
			{
				TextBoxKeyUp( this, e );
			}
		}

		private void textBoxFilter_KeyDown( object sender, KeyEventArgs e )
		{
			if ( e.KeyData == selectAllKey )
			{
				textBoxFilter.SelectAll();
			}
			else
			{
				OnTextBoxKeyDown( e );
			}
		}

		private void textBoxFilter_KeyPress( object sender, KeyPressEventArgs e )
		{
			OnTextBoxKeyPress( e );
		}

		private void textBoxFilter_KeyUp( object sender, KeyEventArgs e )
		{
			OnTextBoxKeyUp( e );
		}

		#endregion

		#region Key Events with ListBoxMatchedItems

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyPressEventHandler ListBoxKeyPress;

		protected virtual void OnListBoxKeyPress( KeyPressEventArgs e )
		{
			if ( ListBoxKeyPress != null )
			{
				ListBoxKeyPress( this, e );
			}
		}

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyEventHandler ListBoxKeyDown;

		protected virtual void OnListBoxKeyDown( KeyEventArgs e )
		{
			if ( ListBoxKeyDown != null )
			{
				ListBoxKeyDown( this, e );
			}
		}

		[Category( "Key" )]
		[Browsable( true )]
		public event KeyEventHandler ListBoxKeyUp;

		protected virtual void OnListBoxKeyUp( KeyEventArgs e )
		{
			if ( ListBoxKeyUp != null )
			{
				ListBoxKeyUp( this, e );
			}
		}

		private void listBoxMatchedItems_KeyDown( object sender, KeyEventArgs e )
		{
			OnListBoxKeyDown( e );
		}

		private void listBoxMatchedItems_KeyPress( object sender, KeyPressEventArgs e )
		{
			OnListBoxKeyPress( e );
		}

		private void listBoxMatchedItems_KeyUp( object sender, KeyEventArgs e )
		{
			OnListBoxKeyUp( e );
		}

		#endregion

		#endregion Key Events

		#region Enter and Leave Events

		[Category( "Focus" )]
		[Browsable( true )]
		public event EventHandler TextBoxEnter;

		protected virtual void OnTextBoxEnter( EventArgs e )
		{
			if ( TextBoxEnter != null )
			{
				TextBoxEnter( this, e );
			}
		}

		[Category( "Focus" )]
		[Browsable( true )]
		public event EventHandler TextBoxLeave;

		protected virtual void OnTextBoxLeave( EventArgs e )
		{
			if ( TextBoxLeave != null )
			{
				TextBoxLeave( this, e );
			}
		}

		[Category( "Focus" )]
		[Browsable( true )]
		public event EventHandler ListBoxEnter;

		protected virtual void OnListBoxEnter( EventArgs e )
		{
			if ( ListBoxEnter != null )
			{
				ListBoxEnter( this, e );
			}
		}

		[Category( "Focus" )]
		[Browsable( true )]
		public event EventHandler ListBoxLeave;

		protected virtual void OnListBoxLeave( EventArgs e )
		{
			if ( ListBoxLeave != null )
			{
				ListBoxLeave( this, e );
			}
		}

		private void textBoxFilter_Enter( object sender, EventArgs e )
		{
			OnTextBoxEnter( e );
		}

		private void textBoxFilter_Leave( object sender, EventArgs e )
		{
			OnTextBoxLeave( e );
		}

		private void listBoxMatchedItems_Enter( object sender, EventArgs e )
		{
			OnListBoxEnter( e );
		}

		private void listBoxMatchedItems_Leave( object sender, EventArgs e )
		{
			OnListBoxLeave( e );
		}

		#endregion

		[Category( "Behavior" )]
		[Browsable( true )]
		public event EventHandler SelectedIndexChanged;

		protected virtual void OnSelectedIndexChanged( EventArgs e )
		{
			if ( SelectedIndexChanged != null )
			{
				SelectedIndexChanged( this, e );
			}
		}

		private void listBoxMatchedItems_SelectedIndexChanged( object sender, EventArgs e )
		{
			OnSelectedIndexChanged( e );
		}

		public bool HasMatchedItems()
		{
			return 0 < listBoxMatchedItems.Items.Count;
		}

		public int GetMatchedItemsCount()
		{
			return listBoxMatchedItems.Items.Count;
		}

		public int GetSelectedItemsCount()
		{
			return listBoxMatchedItems.SelectedItems.Count;
		}

		public void SetSelected( int index, bool value )
		{
			listBoxMatchedItems.SetSelected( index, value );
		}

		public void SelectFirstItem()
		{
			if ( HasMatchedItems() )
			{
				SetSelected( 0, true );
			}
		}

		public bool HasSelectedItems()
		{
			return 0 < listBoxMatchedItems.SelectedItems.Count;
		}

		public void ClearSelected()
		{
			listBoxMatchedItems.ClearSelected();
		}

		public bool FocusToTextBox()
		{
			return textBoxFilter.Focus();
		}

		public bool FocusToListBox()
		{
			return listBoxMatchedItems.Focus();
		}

		public void ClearFilterText()
		{
			textBoxFilter.Clear();
		}

		#region GetItems Methods

		public T GetFirstItem<T>()
		{
			if ( HasMatchedItems() )
			{
				return (T) listBoxMatchedItems.Items[0];
			}

			return default( T );
		}

		public List<T> GetMatchedItems<T>()
		{
			return listBoxMatchedItems.Items.Cast<T>().ToList();
		}

		public T GetSelectedItem<T>()
		{
			if ( listBoxMatchedItems.SelectedItem == null )
			{
				return default( T );
			}

			return (T) listBoxMatchedItems.SelectedItem;
		}

		public List<T> GetSelectedItems<T>()
		{
			return listBoxMatchedItems.SelectedItems.Cast<T>().ToList();
		}

		#endregion

		#region UpdateMatchedItems

		[Browsable( true )]
		public event EventHandler<EventArgs> MatchedItemsUpdating;

		protected virtual void OnMatchedItemsUpdating( EventArgs e )
		{
			if ( MatchedItemsUpdating != null )
			{
				MatchedItemsUpdating( this, e );
			}
		}

		[Browsable( true )]
		public event EventHandler<EventArgs> MatchedItemsUpdated;

		protected virtual void OnMatchedItemsUpdated( EventArgs e )
		{
			if ( MatchedItemsUpdated != null )
			{
				MatchedItemsUpdated( this, e );
			}
		}

		public void UpdateMatchedItems()
		{
			OnMatchedItemsUpdating( EventArgs.Empty );

			DoUpdateMatchedItems();

			OnMatchedItemsUpdated( EventArgs.Empty );
		}

		private void DoUpdateMatchedItems()
		{
			object[] matchedItems = FindMatchedItems();

			Array.Sort( matchedItems, GetMatchedItemsSorter() );

			UpdateListBoxMatchedItems( matchedItems );
		}

		private void UpdateListBoxMatchedItems( object[] matchedItems )
		{
			listBoxMatchedItems.BeginUpdate();

			listBoxMatchedItems.Items.Clear();

			AddItemsToListBoxMatchedItems( matchedItems );

			listBoxMatchedItems.EndUpdate();
		}

		private void AddItemsToListBoxMatchedItems( object[] matchedItems )
		{
			if ( 0 < MaxShowItemsCount )
			{
				IEnumerable<object> items = matchedItems.Take( MaxShowItemsCount );

				listBoxMatchedItems.Items.AddRange( items.ToArray() );
			}
			else
			{
				listBoxMatchedItems.Items.AddRange( matchedItems );
			}
		}

		#region FindMatchedItems

		private object[] FindMatchedItems()
		{
			object[] matchedItems;

			object[] source = EnumerableToArray( GetSource() );

			if ( IsMatchedItem == null )
			{
				matchedItems = Array.FindAll( source, DefaultIsMatchedItem );
			}
			else
			{
				matchedItems = Array.FindAll( source, IsMatchedItem );
			}

			return matchedItems;
		}

		private object[] EnumerableToArray( IEnumerable enumerable )
		{
			List<object> list = new List<object>();

			foreach ( object item in enumerable )
			{
				list.Add( item );
			}

			return list.ToArray();
		}

		protected virtual bool DefaultIsMatchedItem( object item )
		{
			if ( item == null )
			{
				return false;
			}

			return item.ToString().StartsWith( textBoxFilter.Text );
		}

		#endregion

		#endregion
	}
}