using System;
using NUnit.Framework;

namespace Tomham.Tests
{
	[TestFixture]
	public class CounterTest
	{
		private int maxCount = 3;
		private int interval = 2;

		[Test]
		public void Create()
		{
			var counter = new Counter();

			Assert.AreEqual( 1, counter.Interval );
			Assert.AreEqual( int.MaxValue, counter.MaxCount );
		}

		[Test]
		[ExpectedException( typeof ( ArgumentOutOfRangeException ) )]
		public void CreateWithWrongInterval()
		{
			var counter = new Counter( maxCount, 0 );
		}

		[Test]
		[ExpectedException( typeof ( ArgumentOutOfRangeException ) )]
		public void CreateWithWrongMaxCount()
		{
			var counter = new Counter( 0, interval );
		}

		[Test]
		public void Reset()
		{
			var counter = new Counter();

			counter.Update();

			Assert.AreEqual( 1, counter.Count );

			counter.ReSet();

			Assert.AreEqual( 0, counter.Count );
		}

		[Test]
		public void UpdateDontExpectToChangeCount()
		{
			var counter = new Counter( maxCount, interval );

			counter.Update();

			Assert.AreEqual( 0, counter.Count );
		}

		[Test]
		public void UpdateExpectToChangeCount()
		{
			var counter = new Counter( maxCount, interval );

			for ( int i = 0; i < interval; i++ )
			{
				counter.Update();
			}

			Assert.AreEqual( 1, counter.Count );
		}

		[Test]
		public void UpdateWithLoop()
		{
			var counter = new Counter( maxCount, interval );

			counter.Loop = true;

			for ( int i = 0; i < 8; i++ )
			{
				counter.Update();
			}

			Assert.AreEqual( 0, counter.Count );
		}

		[Test]
		public void UpdateWithoutLoop()
		{
			var counter = new Counter( maxCount, interval );

			counter.Loop = false;

			for ( int i = 0; i < 8; i++ )
			{
				counter.Update();
			}

			Assert.AreEqual( maxCount, counter.Count );
			Assert.IsFalse( counter.Enabled );
		}
	}
}